import logging

from django.conf import settings
from waldur_ansible.common import backend as common_backend
from waldur_ansible.jupyter_hub_management import models, constants
from waldur_ansible.jupyter_hub_management.backend import locking_service
from waldur_ansible.python_management.backend import extracted_information_handlers as python_handlers, output_lines_post_processors as python_post_processors, error_handlers as python_error_handlers

from . import additional_extra_args_builders, extracted_information_handlers, error_handlers

logger = logging.getLogger(__name__)


class JupyterHubManagementBackend(common_backend.ManagementRequestsBackend):
    REQUEST_TYPES_PLAYBOOKS_MAP = {
        models.JupyterHubManagementSyncConfigurationRequest: constants.JupyterHubManagementConstants.SYNC_CONFIGURATION,
        models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest: constants.JupyterHubManagementConstants.MAKE_VIRTUAL_ENV_GLOBAL,
        models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest: constants.JupyterHubManagementConstants.MAKE_VIRTUAL_ENV_LOCAL,
        models.JupyterHubManagementDeleteRequest: constants.JupyterHubManagementConstants.DELETE_JUPYTER_HUB,
    }

    REQUEST_TYPES_EXTRA_ARGS_MAP = {
        models.JupyterHubManagementSyncConfigurationRequest: additional_extra_args_builders.build_sync_config_extra_args,
        models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest: additional_extra_args_builders.build_virtual_env_extra_args,
        models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest: additional_extra_args_builders.build_virtual_env_extra_args,
        models.JupyterHubManagementDeleteRequest: additional_extra_args_builders.build_delete_jupyter_hub_extra_args,
    }

    REQUEST_TYPES_POST_PROCESSOR_MAP = {
        models.JupyterHubManagementSyncConfigurationRequest: python_post_processors.NullOutputLinesPostProcessor,
        models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest: python_post_processors.InstalledLibrariesOutputLinesPostProcessor,
        models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest: python_post_processors.InstalledLibrariesOutputLinesPostProcessor,
        models.JupyterHubManagementDeleteRequest: python_post_processors.NullOutputLinesPostProcessor,
    }

    REQUEST_TYPES_HANDLERS_MAP = {
        models.JupyterHubManagementSyncConfigurationRequest: python_handlers.NullExtractedInformationHandler,
        models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest: extracted_information_handlers.JupyterHubVirtualEnvironmentGlobalExtractedInformationHandler,
        models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest: extracted_information_handlers.JupyterHubVirtualEnvironmentLocalExtractedInformationHandler,
        models.JupyterHubManagementDeleteRequest: extracted_information_handlers.JupyterHubManagementDeleteExtractedInformationHandler,
    }

    REQUEST_TYPES_ERROR_HANDLERS_CORRESPONDENCE = {
        models.JupyterHubManagementSyncConfigurationRequest: python_error_handlers.NullErrorHandler,
        models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest: python_error_handlers.NullErrorHandler,
        models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest: python_error_handlers.NullErrorHandler,
        models.JupyterHubManagementDeleteRequest: error_handlers.DeleteRequestErrorHandler,
    }

    LOCKED_FOR_PROCESSING = 'Whole environment or the particular virtual environment ' \
                            'is now being processed, request cannot be executed!'

    def process_jupyter_hub_management_request(self, jupyter_hub_management_request):
        self.process_request(jupyter_hub_management_request)

    def is_processing_allowed(self, request):
        return locking_service.JupyterHubManagementBackendLockingService.is_processing_allowed(request)

    def build_locked_for_processing_message(self, request):
        return JupyterHubManagementBackend.LOCKED_FOR_PROCESSING

    def lock_for_processing(self, request):
        locking_service.JupyterHubManagementBackendLockingService.lock_for_processing(request)

    def handle_on_processing_finished(self, request):
        locking_service.JupyterHubManagementBackendLockingService.handle_on_processing_finished(request)

    def get_playbook_path(self, request):
        return settings.WALDUR_JUPYTER_HUB_MANAGEMENT.get('JUPYTER_MANAGEMENT_PLAYBOOKS_DIRECTORY') \
            + JupyterHubManagementBackend.REQUEST_TYPES_PLAYBOOKS_MAP.get(type(request)) \
            + '.yml'

    def get_user(self, request):
        return request.jupyter_hub_management.python_management.user

    def build_additional_extra_vars(self, request):
        python_management = request.jupyter_hub_management.python_management
        extra_vars = dict(
            instance_public_ip=python_management.instance.external_ips[0] if python_management.instance else None,
            virtual_envs_dir_path=python_management.virtual_envs_dir_path,
            default_system_user=python_management.system_user,
        )

        additional_extra_args_building_function = JupyterHubManagementBackend.REQUEST_TYPES_EXTRA_ARGS_MAP.get(type(request))
        if additional_extra_args_building_function:
            extra_vars.update(additional_extra_args_building_function(request))

        return extra_vars

    def instantiate_line_post_processor_class(self, request):
        lines_post_processor_class = JupyterHubManagementBackend.REQUEST_TYPES_POST_PROCESSOR_MAP.get(type(request))
        return lines_post_processor_class()

    def instantiate_extracted_information_handler_class(self, request):
        extracted_information_handler_class = JupyterHubManagementBackend.REQUEST_TYPES_HANDLERS_MAP.get(type(request))
        return extracted_information_handler_class()

    def instantiate_error_handler_class(self, request):
        extracted_information_handler_class = JupyterHubManagementBackend.REQUEST_TYPES_ERROR_HANDLERS_CORRESPONDENCE.get(type(request))
        return extracted_information_handler_class()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-25 07:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0018_port_runtime_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='port',
            name='service_project_link',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='waldur_vmware.VMwareServiceProjectLink'),
            preserve_default=False,
        ),
    ]



from __future__ import unicode_literals

import base64
from decimal import Decimal
import StringIO

from django.core.exceptions import ValidationError
from django.contrib.auth import get_user_model
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import JSONField as BetterJSONField
from django.core.validators import MinValueValidator, MaxValueValidator, RegexValidator
from django.db import models
from django.db.models import Q
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition, FSMIntegerField
from model_utils import FieldTracker
from model_utils.models import TimeStampedModel, TimeFramedModel
from rest_framework import exceptions as rf_exceptions
import six

from waldur_core.core import models as core_models, utils as core_utils
from waldur_core.core.fields import JSONField
from waldur_core.media.validators import ImageValidator
from waldur_core.logging.loggers import LoggableMixin
from waldur_core.quotas import fields as quotas_fields
from waldur_core.quotas import models as quotas_models
from waldur_core.structure import models as structure_models
from waldur_core.media.models import get_upload_path

from . import managers, plugins
from .attribute_types import ATTRIBUTE_TYPES
from ..common import mixins as common_mixins


@python_2_unicode_compatible
class ServiceProvider(core_models.UuidMixin,
                      core_models.DescribableMixin,
                      structure_models.StructureModel,
                      TimeStampedModel):
    customer = models.OneToOneField(structure_models.Customer, on_delete=models.CASCADE)
    enable_notifications = models.BooleanField(default=True)
    api_secret_code = models.CharField(max_length=255, null=True, blank=True)
    lead_email = models.EmailField(null=True, blank=True,
                                   help_text=_('Email for notification about new request based order items. '
                                               'If this field is set, notifications will be sent.'))
    lead_subject = models.CharField(max_length=255, blank=True,
                                    help_text=_('Notification subject template. '
                                                'Django template variables can be used.'))
    lead_body = models.TextField(blank=True,
                                 help_text=_('Notification body template. '
                                             'Django template variables can be used.'))

    class Permissions(object):
        customer_path = 'customer'

    class Meta(object):
        verbose_name = _('Service provider')

    def __str__(self):
        return six.text_type(self.customer)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-service-provider'

    @property
    def has_active_offerings(self):
        return Offering.objects.filter(customer=self.customer).exclude(state=Offering.States.ARCHIVED).exists()

    def generate_api_secret_code(self):
        self.api_secret_code = core_utils.pwgen()

    def save(self, *args, **kwargs):
        if not self.pk:
            self.generate_api_secret_code()
        super(ServiceProvider, self).save(*args, **kwargs)


@python_2_unicode_compatible
class Category(core_models.UuidMixin,
               quotas_models.QuotaModelMixin,
               TimeStampedModel):
    title = models.CharField(blank=False, max_length=255)
    icon = models.FileField(upload_to='marketplace_category_icons',
                            blank=True,
                            null=True,
                            validators=[ImageValidator])
    description = models.TextField(blank=True)
    backend_id = models.CharField(max_length=255, blank=True)

    class Quotas(quotas_models.QuotaModelMixin.Quotas):
        offering_count = quotas_fields.QuotaField(is_backend=True)

    class Meta(object):
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')
        ordering = ('title',)

    def __str__(self):
        return six.text_type(self.title)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-category'


@python_2_unicode_compatible
class CategoryColumn(models.Model):
    """
    This model is needed in order to render resources table with extra columns.
    Usually each column corresponds to specific resource attribute.
    However, one table column may correspond to several resource attributes.
    In this case custom widget should be specified.
    If attribute field is specified, it is possible to filter and sort resources by it's value.
    """

    class Meta(object):
        ordering = ('category', 'index')

    category = models.ForeignKey(Category, related_name='columns')
    index = models.PositiveSmallIntegerField(help_text=_('Index allows to reorder columns.'))
    title = models.CharField(blank=False, max_length=255,
                             help_text=_('Title is rendered as column header.'))
    attribute = models.CharField(blank=True, max_length=255,
                                 help_text=_('Resource attribute is rendered as table cell.'))
    widget = models.CharField(blank=True, max_length=255,
                              help_text=_('Widget field allows to customise table cell rendering.'))

    def __str__(self):
        return six.text_type(self.title)

    def clean(self):
        if not self.attribute and not self.widget:
            raise ValidationError(_('Either attribute or widget field should be specified.'))


@python_2_unicode_compatible
class Section(TimeStampedModel):
    key = models.CharField(primary_key=True, max_length=255)
    title = models.CharField(blank=False, max_length=255)
    category = models.ForeignKey(Category, related_name='sections')
    is_standalone = models.BooleanField(
        default=False, help_text=_('Whether section is rendered as a separate tab.'))

    def __str__(self):
        return six.text_type(self.title)


InternalNameValidator = RegexValidator('^[a-zA-Z][a-zA-Z0-9_]+$')


@python_2_unicode_compatible
class Attribute(TimeStampedModel):
    key = models.CharField(primary_key=True, max_length=255, validators=[InternalNameValidator])
    title = models.CharField(blank=False, max_length=255)
    section = models.ForeignKey(Section, related_name='attributes')
    type = models.CharField(max_length=255, choices=ATTRIBUTE_TYPES)
    required = models.BooleanField(default=False, help_text=_('A value must be provided for the attribute.'))
    default = BetterJSONField(null=True, blank=True)

    def __str__(self):
        return six.text_type(self.title)


@python_2_unicode_compatible
class AttributeOption(models.Model):
    attribute = models.ForeignKey(Attribute, related_name='options', on_delete=models.CASCADE)
    key = models.CharField(max_length=255, validators=[InternalNameValidator])
    title = models.CharField(max_length=255)

    class Meta(object):
        unique_together = ('attribute', 'key')

    def __str__(self):
        return six.text_type(self.title)


class ScopeMixin(models.Model):
    class Meta(object):
        abstract = True

    content_type = models.ForeignKey(ContentType, null=True, related_name='+')
    object_id = models.PositiveIntegerField(null=True)
    scope = GenericForeignKey('content_type', 'object_id')


class BaseComponent(core_models.DescribableMixin):
    class Meta(object):
        abstract = True

    name = models.CharField(max_length=150,
                            help_text=_('Display name for the measured unit, for example, Floating IP.'))
    type = models.CharField(max_length=50,
                            help_text=_('Unique internal name of the measured unit, for example floating_ip.'),
                            validators=[InternalNameValidator])
    measured_unit = models.CharField(max_length=30,
                                     help_text=_('Unit of measurement, for example, GB.'),
                                     blank=True)


@python_2_unicode_compatible
class CategoryComponent(BaseComponent):
    class Meta(object):
        unique_together = ('type', 'category')

    category = models.ForeignKey(Category, related_name='components')

    def __str__(self):
        return '%s, category: %s' % (self.name, self.category.title)


class CategoryComponentUsage(ScopeMixin):
    component = models.ForeignKey(CategoryComponent)
    date = models.DateField()
    reported_usage = models.PositiveIntegerField(null=True)
    fixed_usage = models.PositiveIntegerField(null=True)
    objects = managers.MixinManager('scope')

    def __str__(self):
        return 'component: %s, date: %s' % (six.text_type(self.component.name), self.date)


@python_2_unicode_compatible
class Offering(core_models.UuidMixin,
               core_models.NameMixin,
               core_models.DescribableMixin,
               quotas_models.QuotaModelMixin,
               structure_models.StructureModel,
               TimeStampedModel,
               ScopeMixin,
               LoggableMixin):

    class States(object):
        DRAFT = 1
        ACTIVE = 2
        PAUSED = 3
        ARCHIVED = 4

        CHOICES = (
            (DRAFT, 'Draft'),
            (ACTIVE, 'Active'),
            (PAUSED, 'Paused'),
            (ARCHIVED, 'Archived'),
        )

    thumbnail = models.FileField(upload_to='marketplace_service_offering_thumbnails',
                                 blank=True,
                                 null=True,
                                 validators=[ImageValidator])
    full_description = models.TextField(blank=True)
    vendor_details = models.TextField(blank=True)
    rating = models.IntegerField(null=True,
                                 validators=[MaxValueValidator(5), MinValueValidator(1)],
                                 help_text=_('Rating is value from 1 to 5.'))
    category = models.ForeignKey(Category, related_name='offerings')
    customer = models.ForeignKey(structure_models.Customer, related_name='+', null=True)
    attributes = BetterJSONField(blank=True, default=dict, help_text=_('Fields describing Category.'))
    options = BetterJSONField(blank=True, default=dict, help_text=_('Fields describing Offering request form.'))
    geolocations = JSONField(default=list, blank=True,
                             help_text=_('List of latitudes and longitudes. For example: '
                                         '[{"latitude": 123, "longitude": 345}, {"latitude": 456, "longitude": 678}]'))

    native_name = models.CharField(max_length=160, default='', blank=True)
    native_description = models.CharField(max_length=500, default='', blank=True)
    terms_of_service = models.TextField(blank=True)

    type = models.CharField(max_length=100)
    state = FSMIntegerField(default=States.DRAFT, choices=States.CHOICES)
    paused_reason = models.TextField(blank=True)

    # If offering is not shared, it is available only to following user categories:
    # 1) staff user;
    # 2) global support user;
    # 3) users with active permission in original customer;
    # 4) users with active permission in allowed customers and nested projects.
    shared = models.BooleanField(default=True, help_text=_('Accessible to all customers.'))
    allowed_customers = models.ManyToManyField(structure_models.Customer, blank=True)

    billable = models.BooleanField(default=True, help_text=_('Purchase and usage is invoiced.'))
    backend_id = models.CharField(max_length=255, blank=True)

    objects = managers.OfferingManager()
    tracker = FieldTracker()

    class Permissions(object):
        customer_path = 'customer'

    class Meta(object):
        verbose_name = _('Offering')

    class Quotas(quotas_models.QuotaModelMixin.Quotas):
        order_item_count = quotas_fields.CounterQuotaField(
            target_models=lambda: [OrderItem],
            path_to_scope='offering',
        )

    @transition(field=state, source=[States.DRAFT, States.PAUSED], target=States.ACTIVE)
    def activate(self):
        pass

    @transition(field=state, source=States.ACTIVE, target=States.PAUSED)
    def pause(self):
        pass

    @transition(field=state, source='*', target=States.ARCHIVED)
    def archive(self):
        pass

    def __str__(self):
        return six.text_type(self.name)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-offering'

    def get_usage_components(self):
        components = self.components.filter(billing_type=OfferingComponent.BillingTypes.USAGE)
        return {component.type: component for component in components}

    @cached_property
    def is_usage_based(self):
        return self.components.filter(billing_type=OfferingComponent.BillingTypes.USAGE).exists()

    @property
    def is_private(self):
        return not self.billable and not self.shared


@python_2_unicode_compatible
class OfferingComponent(common_mixins.ProductCodeMixin, BaseComponent):
    class Meta(object):
        unique_together = ('type', 'offering')

    class BillingTypes(object):
        FIXED = 'fixed'
        USAGE = 'usage'
        ONE_TIME = 'one'
        ON_PLAN_SWITCH = 'few'

        CHOICES = (
            (FIXED, 'Fixed-price'),
            (USAGE, 'Usage-based'),
            (ONE_TIME, 'One-time'),
            # applies fee on resource activation and every time a plan has changed, using pricing of a new plan
            (ON_PLAN_SWITCH, 'One-time on plan switch'),
        )

    class LimitPeriods(object):
        MONTH = 'month'
        TOTAL = 'total'

        CHOICES = (
            (MONTH, 'Maximum monthly - every month service provider '
                    'can report up to the amount requested by user.'),
            (TOTAL, 'Maximum total - SP can report up to the requested '
                    'amount over the whole active state of resource.'),
        )

    offering = models.ForeignKey(Offering, related_name='components')
    parent = models.ForeignKey(CategoryComponent, null=True, blank=True)
    billing_type = models.CharField(choices=BillingTypes.CHOICES,
                                    default=BillingTypes.FIXED,
                                    max_length=5)
    limit_period = models.CharField(choices=LimitPeriods.CHOICES,
                                    blank=True,
                                    null=True,
                                    max_length=5)
    limit_amount = models.IntegerField(blank=True, null=True)
    max_value = models.IntegerField(blank=True, null=True)
    min_value = models.IntegerField(blank=True, null=True)
    disable_quotas = models.BooleanField(
        default=False,
        help_text=_('Do not allow user to specify quotas when offering is provisioned.')
    )

    def validate_amount(self, resource, amount, date):
        if not self.limit_period or not self.limit_amount:
            return

        usages = ComponentUsage.objects.filter(resource=resource, component=self)

        if self.limit_period == OfferingComponent.LimitPeriods.MONTH:
            usages = usages.filter(date=core_utils.month_start(date))

        total = usages.aggregate(models.Sum('usage'))['usage__sum'] or 0

        if total + amount > self.limit_amount:
            raise rf_exceptions.ValidationError(
                _('Total amount exceeds exceeds limit. Total amount: %s, limit: %s.') % (
                    total + amount, self.limit_amount)
            )

    def __str__(self):
        return six.text_type(self.name)


@python_2_unicode_compatible
class Plan(core_models.UuidMixin,
           TimeStampedModel,
           core_models.NameMixin,
           core_models.DescribableMixin,
           common_mixins.UnitPriceMixin,
           common_mixins.ProductCodeMixin,
           ScopeMixin,
           LoggableMixin):
    """
    Plan unit price is computed as a sum of its fixed components'
    price multiplied by component amount when offering with plans
    is created via REST API. Usage-based components don't contribute to plan price.
    It is assumed that plan price is updated manually when plan component is managed via Django ORM.
    """
    offering = models.ForeignKey(Offering, related_name='plans')
    archived = models.BooleanField(default=False, help_text=_('Forbids creation of new resources.'))
    objects = managers.MixinManager('scope')
    backend_id = models.CharField(max_length=255, blank=True)
    max_amount = models.PositiveSmallIntegerField(
        blank=True,
        null=True,
        validators=[MinValueValidator(1)],
        help_text=_('Maximum number of plans that could be active. '
                    'Plan is disabled when maximum amount is reached.')
    )
    tracker = FieldTracker()

    class Meta(object):
        ordering = ('name',)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-plan'

    class Permissions(object):
        customer_path = 'offering__customer'

    def get_estimate(self, limits=None):
        cost = self.unit_price

        if limits:
            available_limits = plugins.manager.get_available_limits(self.offering.type)
            components = self.offering.components.filter(
                Q(billing_type=OfferingComponent.BillingTypes.USAGE) |
                Q(type__in=available_limits)
            )
            components_map = {component.type: component for component in components}
            component_prices = {c.component.type: c.price for c in self.components.all()}
            builtin_components = plugins.manager.get_components(self.offering.type)
            component_factors = {c.type: c.factor for c in builtin_components}
            for key in components_map.keys():
                price = component_prices.get(key, 0)
                limit = limits.get(key, 0)
                factor = component_factors.get(key, 1)
                cost += price * limit / factor

        return cost

    def __str__(self):
        return six.text_type(self.name)

    @property
    def init_price(self):
        return self.sum_components(OfferingComponent.BillingTypes.ONE_TIME)

    @property
    def switch_price(self):
        return self.sum_components(OfferingComponent.BillingTypes.ON_PLAN_SWITCH)

    def sum_components(self, billing_type):
        components = self.components.filter(component__billing_type=billing_type)
        return components.aggregate(sum=models.Sum('price'))['sum'] or 0

    @property
    def has_connected_resources(self):
        return Resource.objects.filter(plan=self).exists()

    @property
    def is_active(self):
        if not self.max_amount:
            return True
        usage = Resource.objects.filter(plan=self).exclude(state=Resource.States.TERMINATED).count()
        return self.max_amount > usage


@python_2_unicode_compatible
class PlanComponent(models.Model):
    class Meta(object):
        unique_together = ('plan', 'component')

    PRICE_MAX_DIGITS = 15
    PRICE_DECIMAL_PLACES = 7

    plan = models.ForeignKey(Plan, related_name='components')
    component = models.ForeignKey(OfferingComponent, related_name='components', null=True)
    amount = models.PositiveIntegerField(default=0)
    price = models.DecimalField(default=0,
                                max_digits=PRICE_MAX_DIGITS,
                                decimal_places=PRICE_DECIMAL_PLACES,
                                validators=[MinValueValidator(Decimal('0'))],
                                verbose_name=_('Price per unit per billing period.'))
    tracker = FieldTracker()

    @property
    def has_connected_resources(self):
        return self.plan.has_connected_resources

    def __str__(self):
        name = self.component and self.component.name
        if name:
            return '%s, plan: %s' % (name, self.plan.name)
        return 'for plan: %s' % self.plan.name


@python_2_unicode_compatible
class Screenshot(core_models.UuidMixin,
                 structure_models.StructureModel,
                 core_models.DescribableMixin,
                 TimeStampedModel,
                 core_models.NameMixin):
    image = models.ImageField(upload_to=get_upload_path)
    thumbnail = models.ImageField(upload_to=get_upload_path, editable=False, null=True)
    offering = models.ForeignKey(Offering, related_name='screenshots')

    class Permissions(object):
        customer_path = 'offering__customer'

    class Meta(object):
        verbose_name = _('Screenshot')

    def __str__(self):
        return six.text_type(self.name)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-screenshot'


class CostEstimateMixin(models.Model):
    class Meta(object):
        abstract = True

    # Cost estimate is computed with respect to fixed plan components and usage-based limits
    cost = models.DecimalField(max_digits=22, decimal_places=10, null=True, blank=True)
    plan = models.ForeignKey(Plan, null=True, blank=True)
    limits = BetterJSONField(blank=True, default=dict)

    def init_cost(self):
        if self.plan:
            self.cost = self.plan.get_estimate(self.limits)


class RequestTypeMixin(CostEstimateMixin):
    class Types(object):
        CREATE = 1
        UPDATE = 2
        TERMINATE = 3

        CHOICES = (
            (CREATE, 'Create'),
            (UPDATE, 'Update'),
            (TERMINATE, 'Terminate'),
        )

    type = models.PositiveSmallIntegerField(choices=Types.CHOICES, default=Types.CREATE)

    class Meta(object):
        abstract = True

    def init_cost(self):
        super(RequestTypeMixin, self).init_cost()
        if self.plan:
            if self.type == RequestTypeMixin.Types.CREATE:
                self.cost += self.plan.init_price
            elif self.type == RequestTypeMixin.Types.UPDATE:
                self.cost += self.plan.switch_price


@python_2_unicode_compatible
class CartItem(core_models.UuidMixin, TimeStampedModel, RequestTypeMixin):
    user = models.ForeignKey(core_models.User, related_name='+', on_delete=models.CASCADE)
    project = models.ForeignKey(structure_models.Project, related_name='+', on_delete=models.CASCADE)
    offering = models.ForeignKey(Offering, related_name='+', on_delete=models.CASCADE)
    attributes = BetterJSONField(blank=True, default=dict)

    class Permissions(object):
        customer_path = 'project__customer'
        project_path = 'project'

    class Meta(object):
        ordering = ('created',)

    def __str__(self):
        return 'user: %s, offering: %s' % (self.user.username, self.offering.name)


@python_2_unicode_compatible
class Order(core_models.UuidMixin, TimeStampedModel, LoggableMixin):
    class States(object):
        REQUESTED_FOR_APPROVAL = 1
        EXECUTING = 2
        DONE = 3
        TERMINATED = 4
        ERRED = 5
        REJECTED = 6

        CHOICES = (
            (REQUESTED_FOR_APPROVAL, 'requested for approval'),
            (EXECUTING, 'executing'),
            (DONE, 'done'),
            (TERMINATED, 'terminated'),
            (ERRED, 'erred'),
            (REJECTED, 'rejected'),
        )

    created_by = models.ForeignKey(core_models.User, related_name='orders')
    approved_by = models.ForeignKey(core_models.User, blank=True, null=True, related_name='+')
    approved_at = models.DateTimeField(editable=False, null=True, blank=True)
    project = models.ForeignKey(structure_models.Project)
    state = FSMIntegerField(default=States.REQUESTED_FOR_APPROVAL, choices=States.CHOICES)
    total_cost = models.DecimalField(max_digits=22, decimal_places=10, null=True, blank=True)
    tracker = FieldTracker()
    _file = models.TextField(blank=True, editable=False)

    class Permissions(object):
        customer_path = 'project__customer'
        project_path = 'project'

    class Meta(object):
        verbose_name = _('Order')
        ordering = ('created',)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-order'

    @transition(field=state, source=States.REQUESTED_FOR_APPROVAL, target=States.EXECUTING)
    def approve(self):
        pass

    @transition(field=state, source=States.EXECUTING, target=States.DONE)
    def complete(self):
        pass

    @transition(field=state, source='*', target=States.TERMINATED)
    def terminate(self):
        pass

    @transition(field=state, source=States.REQUESTED_FOR_APPROVAL, target=States.REJECTED)
    def reject(self):
        pass

    @transition(field=state, source='*', target=States.ERRED)
    def fail(self):
        pass

    def get_approvers(self):
        User = get_user_model()
        users = []

        if settings.WALDUR_MARKETPLACE['NOTIFY_STAFF_ABOUT_APPROVALS']:
            users = User.objects.filter(is_staff=True, is_active=True)

        if settings.WALDUR_MARKETPLACE['OWNER_CAN_APPROVE_ORDER']:
            order_owners = self.project.customer.get_owners()
            users = order_owners if not users else users.union(order_owners)

        if settings.WALDUR_MARKETPLACE['MANAGER_CAN_APPROVE_ORDER']:
            order_managers = self.project.get_users(structure_models.ProjectRole.MANAGER)
            users = order_managers if not users else users.union(order_managers)

        if settings.WALDUR_MARKETPLACE['ADMIN_CAN_APPROVE_ORDER']:
            order_admins = self.project.get_users(structure_models.ProjectRole.ADMINISTRATOR)
            users = order_admins if not users else users.union(order_admins)

        return users and users.distinct()

    @property
    def file(self):
        if not self._file:
            return

        content = base64.b64decode(self._file)
        return StringIO.StringIO(content)

    @file.setter
    def file(self, value):
        self._file = value

    def has_file(self):
        return bool(self._file)

    def get_filename(self):
        return 'marketplace_order_{}.pdf'.format(self.uuid)

    def add_item(self, **kwargs):
        order_item = OrderItem(order=self, **kwargs)
        order_item.clean()
        order_item.init_cost()
        order_item.save()
        return order_item

    def init_total_cost(self):
        self.total_cost = sum(item.cost or 0 for item in self.items.all())

    def get_log_fields(self):
        return (
            'uuid', 'name', 'created', 'modified',
            'created_by', 'approved_by', 'approved_at',
            'project', 'get_state_display', 'total_cost',
        )

    def _get_log_context(self, entity_name):
        context = super(Order, self)._get_log_context(entity_name)
        context['order_items'] = [item._get_log_context('') for item in self.items.all()]
        return context

    def __str__(self):
        return 'project: %s, created by: %s' % (self.project.name, self.created_by.username)


@python_2_unicode_compatible
class Resource(CostEstimateMixin,
               core_models.UuidMixin,
               TimeStampedModel,
               ScopeMixin,
               structure_models.StructureLoggableMixin,
               core_models.NameMixin):
    """
    Core resource is abstract model, marketplace resource is not abstract,
    therefore we don't need to compromise database query efficiency when
    we are getting a list of all resources.

    While migration from ad-hoc resources to marketplace as single entry point is pending,
    the core resource model may continue to be used in plugins and referenced via
    generic foreign key, and marketplace resource is going to be used as consolidated
    model for synchronization with external plugins.

    Eventually it is expected that core resource model is going to be superseded by
    marketplace resource model as a primary mean.
    """
    class States(object):
        CREATING = 1
        OK = 2
        ERRED = 3
        UPDATING = 4
        TERMINATING = 5
        TERMINATED = 6

        CHOICES = (
            (CREATING, 'Creating'),
            (OK, 'OK'),
            (ERRED, 'Erred'),
            (UPDATING, 'Updating'),
            (TERMINATING, 'Terminating'),
            (TERMINATED, 'Terminated'),
        )

    class Permissions(object):
        customer_path = 'project__customer'
        project_path = 'project'

    state = FSMIntegerField(default=States.CREATING, choices=States.CHOICES)
    project = models.ForeignKey(structure_models.Project, on_delete=models.CASCADE)
    offering = models.ForeignKey(Offering, related_name='+', on_delete=models.PROTECT)
    attributes = BetterJSONField(blank=True, default=dict)
    backend_metadata = BetterJSONField(blank=True, default=dict)
    current_usages = BetterJSONField(blank=True, default=dict)
    tracker = FieldTracker()
    objects = managers.MixinManager('scope')

    @property
    def customer(self):
        return self.project.customer

    @transition(field=state, source=[States.ERRED, States.CREATING, States.UPDATING], target=States.OK)
    def set_state_ok(self):
        pass

    @transition(field=state, source='*', target=States.ERRED)
    def set_state_erred(self):
        pass

    @transition(field=state, source='*', target=States.UPDATING)
    def set_state_updating(self):
        pass

    @transition(field=state, source='*', target=States.TERMINATING)
    def set_state_terminating(self):
        pass

    @transition(field=state, source='*', target=States.TERMINATED)
    def set_state_terminated(self):
        pass

    @property
    def backend_uuid(self):
        if self.scope:
            return self.scope.uuid

    @property
    def backend_type(self):
        if self.scope:
            return self.scope.get_scope_type()

    def init_quotas(self):
        if self.limits:
            components_map = self.offering.get_usage_components()
            for key, value in self.limits.items():
                component = components_map.get(key)
                if component:
                    ComponentQuota.objects.create(
                        resource=self,
                        component=component,
                        limit=value
                    )

    def get_log_fields(self):
        return (
            'uuid', 'name', 'project', 'offering', 'created', 'modified',
            'attributes', 'cost', 'plan', 'limits', 'get_state_display',
            'backend_metadata', 'backend_uuid', 'backend_type',
        )

    def __str__(self):
        return six.text_type(self.name)


@python_2_unicode_compatible
class ResourcePlanPeriod(TimeStampedModel, TimeFramedModel, core_models.UuidMixin):
    """
    This model allows to track billing plan for timeframes during resource lifecycle.
    """
    resource = models.ForeignKey(Resource, related_name='+')
    plan = models.ForeignKey(Plan)

    def __str__(self):
        return six.text_type(self.resource.name)

    @property
    def current_components(self):
        now = timezone.now()
        return self.components.filter(billing_period=core_utils.month_start(now))


@python_2_unicode_compatible
class OrderItem(core_models.UuidMixin,
                core_models.ErrorMessageMixin,
                RequestTypeMixin,
                structure_models.StructureLoggableMixin,
                TimeStampedModel):
    class States(object):
        PENDING = 1
        EXECUTING = 2
        DONE = 3
        ERRED = 4
        TERMINATED = 5
        TERMINATING = 6

        CHOICES = (
            (PENDING, 'pending'),
            (EXECUTING, 'executing'),
            (DONE, 'done'),
            (ERRED, 'erred'),
            (TERMINATED, 'terminated'),
            (TERMINATING, 'terminating'),
        )

        TERMINAL_STATES = {DONE, ERRED}

    order = models.ForeignKey(Order, related_name='items')
    offering = models.ForeignKey(Offering)
    attributes = BetterJSONField(blank=True, default=dict)
    old_plan = models.ForeignKey(Plan, related_name='+', null=True, blank=True)
    resource = models.ForeignKey(Resource, null=True, blank=True)
    state = FSMIntegerField(default=States.PENDING, choices=States.CHOICES)
    activated = models.DateTimeField(_('activation date'), null=True, blank=True)
    tracker = FieldTracker()

    class Permissions(object):
        customer_path = 'order__project__customer'
        project_path = 'order__project'

    class Meta(object):
        verbose_name = _('Order item')
        ordering = ('created',)

    @classmethod
    def get_url_name(cls):
        return 'marketplace-order-item'

    @transition(field=state, source=[States.PENDING, States.ERRED], target=States.EXECUTING)
    def set_state_executing(self):
        pass

    @transition(field=state, source=States.EXECUTING, target=States.DONE)
    def set_state_done(self):
        self.activated = timezone.now()
        self.save()

    @transition(field=state, source='*', target=States.ERRED)
    def set_state_erred(self):
        pass

    @transition(field=state, source='*', target=States.TERMINATED)
    def set_state_terminated(self):
        pass

    @transition(field=state, source=[States.PENDING, States.EXECUTING], target=States.TERMINATING)
    def set_state_terminating(self):
        pass

    def clean(self):
        offering = self.offering
        customer = self.order.project.customer

        if offering.shared:
            return

        if offering.customer == customer:
            return

        if offering.allowed_customers.filter(pk=customer.pk).exists():
            return

        raise ValidationError(
            _('Offering "%s" is not allowed in organization "%s".') % (offering.name, customer.name)
        )

    def get_log_fields(self):
        return (
            'uuid', 'created', 'modified',
            'cost', 'limits', 'attributes',
            'offering', 'resource', 'plan',
            'get_state_display', 'get_type_display',
        )

    @property
    def safe_attributes(self):
        """
        Get attributes excluding secret attributes, such as username and password.
        """
        secret_attributes = plugins.manager.get_secret_attributes(self.offering.type)
        attributes = self.attributes or {}
        return {
            key: attributes[key]
            for key in attributes.keys()
            if key not in secret_attributes
        }

    def __str__(self):
        return 'type: %s, created_by: %s' % (self.get_type_display(), self.order.created_by)


@python_2_unicode_compatible
class ComponentQuota(models.Model):
    resource = models.ForeignKey(Resource, related_name='quotas')
    component = models.ForeignKey(OfferingComponent,
                                  limit_choices_to={'billing_type': OfferingComponent.BillingTypes.USAGE})
    limit = models.PositiveIntegerField(default=-1)
    usage = models.PositiveIntegerField(default=0)

    class Meta:
        unique_together = ('resource', 'component')

    def __str__(self):
        return 'resource: %s, component: %s' % (self.resource.name, self.component.name)


@python_2_unicode_compatible
class ComponentUsage(TimeStampedModel,
                     core_models.DescribableMixin,
                     core_models.UuidMixin):
    resource = models.ForeignKey(Resource, related_name='usages')
    component = models.ForeignKey(OfferingComponent,
                                  limit_choices_to={'billing_type': OfferingComponent.BillingTypes.USAGE})
    usage = models.PositiveIntegerField(default=0)
    date = models.DateTimeField()
    plan_period = models.ForeignKey('ResourcePlanPeriod', related_name='components', null=True)
    billing_period = models.DateField()

    tracker = FieldTracker()

    class Meta:
        unique_together = ('resource', 'component', 'plan_period', 'billing_period')

    def __str__(self):
        return 'resource: %s, component: %s' % (self.resource.name, self.component.name)


@python_2_unicode_compatible
class AggregateResourceCount(ScopeMixin):
    """
    This model allows to count current number of project or customer resources by category.
    """
    category = models.ForeignKey(Category, related_name='+')
    count = models.PositiveIntegerField(default=0)
    objects = managers.MixinManager('scope')

    class Meta:
        unique_together = ('category', 'content_type', 'object_id')

    def __str__(self):
        return six.text_type(self.category.title)


@python_2_unicode_compatible
class OfferingFile(core_models.UuidMixin,
                   core_models.NameMixin,
                   structure_models.StructureModel,
                   TimeStampedModel):
    offering = models.ForeignKey(Offering, related_name='files')
    file = models.FileField(upload_to='offering_files')

    class Permissions(object):
        customer_path = 'offering__customer'

    @classmethod
    def get_url_name(cls):
        return 'marketplace-offering-file'

    def __str__(self):
        return 'offering: %s' % self.offering



from __future__ import unicode_literals

import logging

from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition, FSMIntegerField
from model_utils.models import TimeStampedModel

from waldur_core.core.fields import JSONField
from waldur_core.core.models import UuidMixin, ErrorMessageMixin, BackendModelMixin
from waldur_core.logging.loggers import LoggableMixin
from waldur_core.structure.models import Customer
from . import backend

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Payment(LoggableMixin, TimeStampedModel, UuidMixin, ErrorMessageMixin):
    class Meta(object):
        ordering = ['-modified']

    class Permissions(object):
        customer_path = 'customer'

    class States(object):
        INIT = 0
        CREATED = 1
        APPROVED = 2
        CANCELLED = 3
        ERRED = 4

    STATE_CHOICES = (
        (States.INIT, 'Initial'),
        (States.CREATED, 'Created'),
        (States.APPROVED, 'Approved'),
        (States.ERRED, 'Erred'),
    )

    state = FSMIntegerField(default=States.INIT, choices=STATE_CHOICES)

    customer = models.ForeignKey(Customer)
    amount = models.DecimalField(max_digits=9, decimal_places=2)
    tax = models.DecimalField(max_digits=9, decimal_places=2, default=0)

    # Payment ID is persistent identifier of payment
    backend_id = models.CharField(max_length=255, null=True)

    # Token is temporary identifier of payment
    token = models.CharField(max_length=255, null=True)

    # URL is fetched from backend
    approval_url = models.URLField()

    def __str__(self):
        return "%s %.2f %s" % (self.modified, self.amount, self.customer.name)

    def get_backend(self):
        return backend.PaypalBackend()

    @classmethod
    def get_url_name(cls):
        return 'paypal-payment'

    def get_log_fields(self):
        return ('uuid', 'customer', 'amount', 'modified', 'status')

    @transition(field=state, source=States.INIT, target=States.CREATED)
    def set_created(self):
        pass

    @transition(field=state, source=States.CREATED, target=States.APPROVED)
    def set_approved(self):
        pass

    @transition(field=state, source=States.CREATED, target=States.CANCELLED)
    def set_cancelled(self):
        pass

    @transition(field=state, source='*', target=States.ERRED)
    def set_erred(self):
        pass


@python_2_unicode_compatible
class Invoice(LoggableMixin, UuidMixin, BackendModelMixin):
    class Meta(object):
        ordering = ['-invoice_date']

    class Permissions(object):
        customer_path = 'customer'

    class States(object):
        DRAFT = 'DRAFT'
        SENT = 'SENT'
        PAID = 'PAID'
        MARKED_AS_PAID = 'MARKED_AS_PAID'
        CANCELLED = 'CANCELLED'
        REFUNDED = 'REFUNDED'
        PARTIALLY_REFUNDED = 'PARTIALLY_REFUNDED'
        MARKED_AS_REFUNDED = 'MARKED_AS_REFUNDED'
        UNPAID = 'UNPAID'
        PAYMENT_PENDING = 'PAYMENT_PENDING'

        CHOICES = (
            (DRAFT, _('Draft')),
            (SENT, _('Sent')),
            (PAID, _('Paid')),
            (MARKED_AS_PAID, _('Marked as paid')),
            (CANCELLED, _('Cancelled')),
            (REFUNDED, _('Refunded')),
            (PARTIALLY_REFUNDED, _('Partially refunded')),
            (MARKED_AS_REFUNDED, _('Marked as refunded')),
            (UNPAID, _('Unpaid')),
            (PAYMENT_PENDING, _('Payment pending')),
        )

    customer = models.ForeignKey(Customer, related_name='paypal_invoices')
    state = models.CharField(max_length=30, choices=States.CHOICES, default=States.DRAFT)
    invoice_date = models.DateField()
    end_date = models.DateField()
    pdf = models.FileField(upload_to='paypal-invoices', blank=True, null=True)
    number = models.CharField(max_length=30)
    tax_percent = models.DecimalField(default=0, max_digits=4, decimal_places=2,
                                      validators=[MinValueValidator(0), MaxValueValidator(100)])
    backend_id = models.CharField(max_length=128, blank=True)
    issuer_details = JSONField(default=dict, blank=True, help_text=_('Stores data about invoice issuer'))
    payment_details = JSONField(default=dict, blank=True, help_text=_('Stores data about customer payment details'))
    month = models.PositiveSmallIntegerField(validators=[MinValueValidator(1), MaxValueValidator(12)])
    year = models.PositiveSmallIntegerField()

    def get_backend(self):
        return backend.PaypalBackend()

    @classmethod
    def get_backend_fields(cls):
        return super(Invoice, cls).get_backend_fields() + (
            'state', 'issuer_details', 'number', 'payment_details', 'backend_id')

    @classmethod
    def get_url_name(cls):
        return 'paypal-invoice'

    @property
    def file_name(self):
        return '{}-invoice-{}.pdf'.format(self.invoice_date.strftime('%Y-%m-%d'), self.pk)

    @property
    def total(self):
        return self.price + self.tax

    @property
    def price(self):
        return sum(item.price for item in self.items.all())

    @property
    def tax(self):
        return self.price * self.tax_percent / 100

    def get_log_fields(self):
        return ('uuid', 'customer', 'total', 'invoice_date', 'end_date')

    def __str__(self):
        return "Invoice #%s" % self.number or self.id


class InvoiceItem(models.Model):
    class Meta(object):
        ordering = ['invoice', '-start']

    class UnitsOfMeasure(object):
        QUANTITY = 'QUANTITY'
        HOURS = 'HOURS'
        AMOUNT = 'AMOUNT'

        CHOICES = ((QUANTITY, _('Quantity')), (HOURS, _('Hours')), (AMOUNT, _('Amount')))

    invoice = models.ForeignKey(Invoice, related_name='items')
    price = models.DecimalField(max_digits=9, decimal_places=2)
    tax = models.DecimalField(max_digits=9, decimal_places=2, default=0)
    unit_price = models.DecimalField(max_digits=9, decimal_places=2)
    quantity = models.PositiveIntegerField(default=0)
    unit_of_measure = models.CharField(max_length=30, choices=UnitsOfMeasure.CHOICES, default=UnitsOfMeasure.HOURS)
    name = models.CharField(max_length=255)
    start = models.DateTimeField(null=True)
    end = models.DateTimeField(null=True)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-01-22 12:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0053_add_project_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='address',
            field=models.CharField(blank=True, max_length=300),
        ),
        migrations.AddField(
            model_name='customer',
            name='bank_account',
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name='customer',
            name='bank_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='customer',
            name='postal',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AddField(
            model_name='customer',
            name='type',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='customer',
            name='accounting_start_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Start date of accounting'),
        ),
        migrations.AddField(
            model_name='customer',
            name='default_tax_percent',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=4,
                                      validators=[django.core.validators.MinValueValidator(0),
                                                  django.core.validators.MaxValueValidator(100)]),
        ),
    ]



import base64
from calendar import timegm
from datetime import datetime
import os
import tempfile

from django.apps import apps
from django.conf import settings
from django.http import HttpResponse
import jwt
from rest_framework.exceptions import ValidationError
from rest_framework.generics import get_object_or_404
from rest_framework.reverse import reverse

from waldur_core.core import utils
from waldur_core.core.models import User
from waldur_core.structure.managers import filter_queryset_for_user


def encode_attachment_token(user_uuid, obj, field):
    max_age = settings.WALDUR_CORE['ATTACHMENT_LINK_MAX_AGE']
    dt = datetime.utcnow() + max_age
    expires_at = timegm(dt.utctimetuple())
    payload = {
        'usr': user_uuid,
        'ct': str(obj._meta),
        'id': obj.uuid.hex,
        'field': field,
        'exp': expires_at,
    }
    return utils.encode_jwt_token(payload)


def decode_attachment_token(token):
    try:
        data = utils.decode_jwt_token(token)
    except jwt.exceptions.InvalidTokenError:
        raise ValidationError('Bad signature.')

    if not isinstance(data, dict):
        raise ValidationError('Bad token data.')

    user_uuid = data.get('usr')
    content_type = data.get('ct')
    object_uuid = data.get('id')
    field = data.get('field')

    if not user_uuid:
        raise ValidationError('User UUID is not provided.')

    if not content_type:
        raise ValidationError('Content type is not provided.')

    if not object_uuid:
        raise ValidationError('Object UUID is not provided.')

    if not field:
        raise ValidationError('Field is not provided.')

    return user_uuid, content_type, object_uuid, field


def encode_protected_url(obj, field, request=None, user_uuid=None):
    """
    Returns URL with secure token for media download.
    :param obj: database model object which has file field, for example, customer or invoice.
    :param field: field name inside of database model object, for example, image or file
    :param request: authenticated HTTP request
    :param user_uuid: user UUID, it should be specified if request is not specified
    :return: URL with token.
    """
    if not user_uuid:
        user_uuid = request.user.uuid.hex
    token = encode_attachment_token(user_uuid, obj, field)
    return reverse('media-download', request=request, kwargs={'token': token})


def get_file_from_token(token):
    user_uuid, content_type, object_uuid, field = decode_attachment_token(token)
    user = get_object_or_404(User, uuid=user_uuid)
    if not user.is_active:
        raise ValidationError('User is not active.')
    queryset = apps.get_model(content_type).objects.all()
    if hasattr(queryset, 'filter_for_user'):
        queryset = queryset.filter_for_user(user)
    else:
        queryset = filter_queryset_for_user(queryset, user)
    obj = get_object_or_404(queryset, uuid=object_uuid)
    return getattr(obj, field, None)


def send_file(file):
    _, file_name = os.path.split(file.path)
    response = HttpResponse()
    response['Content-Disposition'] = 'attachment; filename=' + file_name
    response['X-Accel-Redirect'] = file.url
    return response


def dummy_image(filetype='gif'):
    """ Generate empty image in temporary file for testing """
    # 1x1px Transparent GIF
    GIF = 'R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7'
    tmp_file = tempfile.NamedTemporaryFile(suffix='.%s' % filetype)
    tmp_file.write(base64.b64decode(GIF))
    return open(tmp_file.name, 'rb')



from django.contrib.contenttypes.models import ContentType

from waldur_core.core.utils import DryRunCommand
from waldur_mastermind.marketplace.models import Resource
from waldur_mastermind.marketplace_openstack import utils
from waldur_openstack.openstack.models import Tenant


class Command(DryRunCommand):
    help = """Import OpenStack tenant quotas to marketplace."""

    def handle(self, dry_run, *args, **options):
        ct = ContentType.objects.get_for_model(Tenant)
        for resource in Resource.objects.filter(content_type=ct):
            utils.import_usage(resource)
            utils.import_limits(resource)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-05-05 15:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0048_remove_balance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='abbreviation',
            field=models.CharField(blank=True, max_length=12),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_user_agreement_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='competence',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AddField(
            model_name='user',
            name='preferred_language',
            field=models.CharField(max_length=10, blank=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='phone_number',
            field=models.CharField(max_length=255, verbose_name='phone number', blank=True),
        ),
    ]



import subprocess  # nosec


def subprocess_output_iterator(command, env, **kwargs):
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True, bufsize=1, env=env, **kwargs)  # nosec
    for stdout_line in iter(process.stdout.readline, ""):
        yield stdout_line
    process.stdout.close()
    return_code = process.wait()
    if return_code:
        raise subprocess.CalledProcessError(return_code, command)



from django.conf import settings as django_settings
from rest_framework import exceptions

from waldur_core.structure import models as structure_models
from waldur_core.structure import permissions as structure_permissions


def can_register_service_provider(request, customer):
    if request.user.is_staff:
        return
    if not django_settings.WALDUR_MARKETPLACE['OWNER_CAN_REGISTER_SERVICE_PROVIDER']:
        raise exceptions.PermissionDenied()
    structure_permissions.is_owner(request, None, customer)


def check_availability_of_auto_approving(items, user, project):
    if user.is_staff:
        return True

    # Skip approval of private offering for project users
    if all(item.offering.is_private for item in items):
        return structure_permissions._has_admin_access(user, project)

    return user_can_approve_order(user, project)


def user_can_approve_order_permission(request, view, order=None):
    if not order:
        return

    user = request.user
    if user_can_approve_order(user, order.project):
        return

    raise exceptions.PermissionDenied()


def user_can_approve_order(user, project):
    if user.is_staff:
        return True

    if django_settings.WALDUR_MARKETPLACE['OWNER_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_owner_access(user, project.customer):
        return True

    if django_settings.WALDUR_MARKETPLACE['MANAGER_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_manager_access(user, project):
        return True

    if django_settings.WALDUR_MARKETPLACE['ADMIN_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_admin_access(user, project):
        return True

    return False


def user_can_reject_order(request, view, order=None):
    if not order:
        return

    user = request.user

    if user.is_staff:
        return

    if user == order.created_by:
        return

    if structure_permissions._has_admin_access(user, order.project):
        return

    raise exceptions.PermissionDenied()


def user_can_list_importable_resources(request, view, offering=None):
    if not offering:
        return

    user = request.user
    if user.is_staff:
        return

    if offering.shared:
        raise exceptions.PermissionDenied('Import is limited to staff for shared offerings.')

    owned_customers = set(structure_models.Customer.objects.all().filter(
        permissions__user=user,
        permissions__is_active=True,
        permissions__role=structure_models.CustomerRole.OWNER
    ).distinct())

    if offering.customer not in owned_customers and not (offering.allowed_customers & owned_customers):
        raise exceptions.PermissionDenied('Import is limited to owners for private offerings.')



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel
from waldur_ansible.common import models as common_models

from waldur_core.core import models as core_models, fields as core_fields
from waldur_core.core.validators import validate_name
from waldur_core.structure import models as structure_models

User = get_user_model()


class PythonManagement(common_models.UuidStrMixin, TimeStampedModel, common_models.ApplicationModel):
    user = models.ForeignKey(User, related_name='+')
    virtual_envs_dir_path = models.CharField(max_length=255)
    python_version = models.CharField(max_length=10)
    system_user = models.CharField(max_length=255, null=True)

    instance_content_type = models.ForeignKey(ContentType, null=True, related_name='+')
    instance_object_id = models.PositiveIntegerField(null=True)
    instance = GenericForeignKey('instance_content_type', 'instance_object_id')

    project = models.ForeignKey(structure_models.Project, null=True, related_name='+')

    class Permissions:
        project_path = 'project'
        customer_path = 'project__customer'

    class Meta:
        unique_together = (('instance_content_type', 'instance_object_id', 'virtual_envs_dir_path', 'system_user'),)

    @staticmethod
    def get_url_name():
        return 'python_management'

    def __str__(self):
        return '%s: %s' % (self.__class__.__name__, self.uuid.hex)


class VirtualEnvironment(common_models.UuidStrMixin, core_models.NameMixin, models.Model):
    python_management = models.ForeignKey(PythonManagement, on_delete=models.CASCADE, related_name='virtual_environments')
    jupyter_hub_global = models.BooleanField(default=False)


class InstalledLibrary(common_models.UuidStrMixin, core_models.NameMixin, models.Model):
    virtual_environment = models.ForeignKey(VirtualEnvironment, on_delete=models.CASCADE, related_name='installed_libraries')
    version = models.CharField(max_length=255)


class PythonManagementRequest(common_models.UuidStrMixin, core_models.StateMixin, TimeStampedModel, common_models.OutputMixin):
    python_management = models.ForeignKey(PythonManagement, on_delete=models.CASCADE, related_name='+')

    class Meta(object):
        abstract = True

    def get_backend(self):
        from waldur_ansible.python_management.backend.python_management_backend import PythonManagementBackend
        return PythonManagementBackend()


class PythonManagementInitializeRequest(PythonManagementRequest):

    # holds sychronization_requests One-To-Many relation

    def get_backend(self):
        from waldur_ansible.python_management.backend.python_management_backend import PythonManagementInitializationBackend
        return PythonManagementInitializationBackend()


class VirtualEnvMixin(models.Model):
    virtual_env_name = models.CharField(max_length=255)

    class Meta(object):
        abstract = True


class PythonManagementSynchronizeRequest(VirtualEnvMixin, PythonManagementRequest):
    libraries_to_install = core_fields.JSONField(default=list, help_text=_('List of libraries to install'), blank=True)
    libraries_to_remove = core_fields.JSONField(default=list, help_text=_('List of libraries to remove'), blank=True)
    initialization_request = models.ForeignKey(PythonManagementInitializeRequest, related_name="sychronization_requests", null=True)


class PythonManagementDeleteRequest(PythonManagementRequest):
    pass


class PythonManagementDeleteVirtualEnvRequest(VirtualEnvMixin, PythonManagementRequest):
    pass


class PythonManagementFindVirtualEnvsRequest(PythonManagementRequest):
    pass


class PythonManagementFindInstalledLibrariesRequest(VirtualEnvMixin, PythonManagementRequest):
    pass


class CachedRepositoryPythonLibrary(common_models.UuidStrMixin):
    name = models.CharField(max_length=255, validators=[validate_name], db_index=True)



import logging
import urlparse

from django.conf import settings as django_settings
from django.utils import timezone
import lxml.etree  # nosec
import requests


logger = logging.getLogger(__name__)


class ClientError(Exception):
    pass


class ResponseParseError(ClientError):
    pass


class ResponseStatusError(ClientError):
    pass


class RequestError(ClientError):
    def __init__(self, message, response=None):
        super(RequestError, self).__init__(message)
        self.response = response


class UnknownStatusError(ResponseParseError):
    pass


class Response(object):
    ns_namespace = 'http://uri.etsi.org/TS102204/v1.1.2#'

    def __init__(self, content):
        etree = lxml.etree.fromstring(content)  # nosec
        self.init_response_attributes(etree)

    def init_response_attributes(self, etree):
        """ Define response attributes based on valimo request content """
        raise NotImplementedError


class Request(object):
    url = NotImplemented
    template = NotImplemented
    response_class = NotImplemented
    settings = getattr(django_settings, 'WALDUR_AUTH_VALIMO', {})

    @classmethod
    def execute(cls, **kwargs):
        url = cls._get_url()
        headers = {
            'content-type': 'text/xml',
            'SOAPAction': url,
        }
        data = cls.template.strip().format(
            AP_ID=cls.settings['AP_ID'],
            AP_PWD=cls.settings['AP_PWD'],
            Instant=cls._format_datetime(timezone.now()),
            DNSName=cls.settings['DNSName'],
            **kwargs
        )
        cert = (cls.settings['cert_path'], cls.settings['key_path'])
        # TODO: add verification
        logger.debug('Executing POST request to %s with data:\n %s \nheaders: %s', url, data, headers)
        response = requests.post(url, data=data, headers=headers, cert=cert, verify=cls.settings['verify_ssl'])
        if response.ok:
            return cls.response_class(response.content)
        else:
            message = 'Failed to execute POST request against %s endpoint. Response [%s]: %s' % (
                url, response.status_code, response.content)
            raise RequestError(message, response)

    @classmethod
    def _format_datetime(cls, d):
        return d.strftime('%Y-%m-%dT%H:%M:%S.000Z')

    @classmethod
    def _format_transaction_id(cls, transaction_id):
        return ('_' + transaction_id)[:32]  # such formation is required by server.

    @classmethod
    def _get_url(cls):
        return urlparse.urljoin(cls.settings['URL'], cls.url)


class SignatureResponse(Response):

    def init_response_attributes(self, etree):
        try:
            self.backend_transaction_id = etree.xpath('//MSS_SignatureResp')[0].attrib['MSSP_TransID']
            self.status = etree.xpath('//ns6:StatusCode', namespaces={'ns6': self.ns_namespace})[0].attrib['Value']
        except (IndexError, KeyError, lxml.etree.XMLSchemaError) as e:
            raise ResponseParseError('Cannot parse signature response: %s. Response content: %s' % (
                e, lxml.etree.tostring(etree)))


class SignatureRequest(Request):
    url = '/MSSP/services/MSS_Signature'
    template = """
        <?xml version="1.0" encoding="UTF-8"?>
        <soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope"
                          xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                          xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
            <soapenv:Body>
                <MSS_Signature xmlns="">
                <MSS_SignatureReq MajorVersion="1" MessagingMode="{MessagingMode}" MinorVersion="1" TimeOut="300">
                    <ns1:AP_Info AP_ID="{AP_ID}" AP_PWD="{AP_PWD}" AP_TransID="{AP_TransID}"
                                 Instant="{Instant}" xmlns:ns1="http://uri.etsi.org/TS102204/v1.1.2#"/>
                    <ns2:MSSP_Info xmlns:ns2="http://uri.etsi.org/TS102204/v1.1.2#">
                        <ns2:MSSP_ID>
                            <ns2:DNSName>{DNSName}</ns2:DNSName>
                        </ns2:MSSP_ID>
                    </ns2:MSSP_Info>
                    <ns3:MobileUser xmlns:ns3="http://uri.etsi.org/TS102204/v1.1.2#">
                        <ns3:MSISDN>{MSISDN}</ns3:MSISDN>
                    </ns3:MobileUser>
                    <ns4:DataToBeSigned Encoding="UTF-8" MimeType="text/plain" xmlns:ns4="http://uri.etsi.org/TS102204/v1.1.2#">
                        {DataToBeSigned}
                    </ns4:DataToBeSigned>
                    <ns5:SignatureProfile xmlns:ns5="http://uri.etsi.org/TS102204/v1.1.2#">
                        <ns5:mssURI>{SignatureProfile}</ns5:mssURI>
                    </ns5:SignatureProfile>
                    <ns6:MSS_Format xmlns:ns6="http://uri.etsi.org/TS102204/v1.1.2#">
                        <ns6:mssURI>http://uri.etsi.org/TS102204/v1.1.2#PKCS7</ns6:mssURI>
                    </ns6:MSS_Format>
                </MSS_SignatureReq>
                </MSS_Signature>
            </soapenv:Body>
        </soapenv:Envelope>
    """
    response_class = SignatureResponse

    @classmethod
    def execute(cls, transaction_id, phone, message):
        kwargs = {
            'MessagingMode': 'asynchClientServer',
            'AP_TransID': cls._format_transaction_id(transaction_id),
            'MSISDN': phone,
            'DataToBeSigned': '%s %s' % (cls.settings['message_prefix'], message),
            'SignatureProfile': cls.settings['SignatureProfile']
        }
        return super(SignatureRequest, cls).execute(**kwargs)


class Statuses(object):
    OK = 'OK'
    PROCESSING = 'Processing'
    ERRED = 'Erred'

    @classmethod
    def map(cls, status_code):
        if status_code == '502':
            return cls.OK
        elif status_code == '504':
            return cls.PROCESSING
        else:
            raise UnknownStatusError('Received unsupported status in response: %s' % status_code)


class StatusResponse(Response):

    def init_response_attributes(self, etree):
        try:
            status_code = etree.xpath('//ns5:StatusCode', namespaces={'ns5': self.ns_namespace})[0].attrib['Value']
        except (IndexError, KeyError, lxml.etree.XMLSchemaError) as e:
            raise ResponseParseError('Cannot parse status response: %s. Response content: %s' % (
                e, lxml.etree.tostring(etree)))
        self.status = Statuses.map(status_code)

        try:
            civil_number_tag = etree.xpath('//ns4:UserIdentifier', namespaces={'ns4': self.ns_namespace})[0]
        except IndexError:
            # civil number tag does not exist - this is possible if request is still processing
            return
        else:
            try:
                self.civil_number = civil_number_tag.text.split('=')[1]
            except IndexError:
                raise ResponseParseError('Cannot get civil_number from tag text: %s' % civil_number_tag.text)


class ErredStatusResponse(Response):
    soapenv_namespace = 'http://www.w3.org/2003/05/soap-envelope'

    def init_response_attributes(self, etree):
        self.status = Statuses.ERRED
        try:
            self.details = etree.xpath('//soapenv:Text', namespaces={'soapenv': self.soapenv_namespace})[0].text
        except (IndexError, lxml.etree.XMLSchemaError) as e:
            raise ResponseParseError('Cannot parse error status response: %s. Response content: %s' % (
                e, lxml.etree.tostring(etree)))


class StatusRequest(Request):
    url = '/MSSP/services/MSS_StatusPort'
    template = """
        <?xml version="1.0" encoding="UTF-8"?>
        <soapenv:Envelope xmlns:soapenv="http://www.w3.org/2003/05/soap-envelope"
                              xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                              xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
            <soapenv:Body>
            <MSS_StatusQuery xmlns="">
            <MSS_StatusReq MajorVersion="1" MinorVersion="1">
                <ns1:AP_Info AP_ID="{AP_ID}" AP_PWD="{AP_PWD}" AP_TransID="{AP_TransID}"
                                 Instant="{Instant}" xmlns:ns1="http://uri.etsi.org/TS102204/v1.1.2#"/>
                <ns2:MSSP_Info xmlns:ns2="http://uri.etsi.org/TS102204/v1.1.2#">
                    <ns2:MSSP_ID>
                        <ns2:DNSName>{DNSName}</ns2:DNSName>
                    </ns2:MSSP_ID>
                </ns2:MSSP_Info>
                <ns3:MSSP_TransID xmlns:ns3="http://uri.etsi.org/TS102204/v1.1.2#">{MSSP_TransID}</ns3:MSSP_TransID>
            </MSS_StatusReq>
            </MSS_StatusQuery>
            </soapenv:Body>
        </soapenv:Envelope>
    """
    response_class = StatusResponse

    @classmethod
    def execute(cls, transaction_id, backend_transaction_id):
        kwargs = {
            'AP_TransID': cls._format_transaction_id(transaction_id),
            'MSSP_TransID': backend_transaction_id,
        }
        try:
            return super(StatusRequest, cls).execute(**kwargs)
        except RequestError as e:
            # If request was timed out or user canceled login - Valimo would return response with status 500
            return ErredStatusResponse(e.response.content)



from __future__ import unicode_literals

from django.core.management.base import BaseCommand

from ... import utils


class Command(BaseCommand):
    help_text = 'Synchronize SAML2 identity providers.'

    def handle(self, *args, **options):
        utils.sync_providers()
        self.stdout.write('SAML2 providers have been successfully synchronized.')



from django.conf import settings as django_settings

from waldur_core.core import filters as core_filters


class VpcExternalFilter(core_filters.BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if request.user.is_staff:
            return queryset
        if not django_settings.WALDUR_CORE['ONLY_STAFF_MANAGES_SERVICES']:
            return queryset
        category_uuid = django_settings.WALDUR_MARKETPLACE_OPENSTACK['TENANT_CATEGORY_UUID']
        return queryset.exclude(category__uuid=category_uuid)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-14 08:43
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('marketplace', '0043_shared_offering'),
    ]

    operations = [
        migrations.CreateModel(
            name='CartItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('attributes', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('limits', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('offering', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='marketplace.Offering')),
                ('plan', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marketplace.Plan')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('created',),
            },
        ),
    ]



from django.conf import settings
from django.core.signing import TimestampSigner, BadSignature
from django.db import transaction
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from waldur_core.core import models as core_models
from waldur_core.core import utils as core_utils
from waldur_core.core.utils import pwgen
from waldur_core.structure.models import ProjectRole
from waldur_core.users import models
from waldur_freeipa.backend import FreeIPABackend
from waldur_freeipa.models import Profile
from waldur_freeipa import tasks
from python_freeipa import exceptions as freeipa_exceptions

from waldur_freeipa.utils import generate_username


def get_invitation_context(invitation, sender):
    if invitation.project_role is not None:
        context = dict(type=_('project'), name=invitation.project.name)
        role_prefix = _('project') if invitation.project_role == ProjectRole.MANAGER else _('system')
        context['role'] = '%s %s' % (role_prefix, invitation.get_project_role_display())

    else:
        context = dict(
            type=_('organization'),
            name=invitation.customer.name,
            role=invitation.get_customer_role_display()
        )

    context['sender'] = sender
    context['invitation'] = invitation
    return context


def get_invitation_token(invitation, user):
    signer = TimestampSigner()
    payload = '%s.%s' % (user.uuid.hex, invitation.uuid.hex)
    return signer.sign(payload)


def parse_invitation_token(token):
    signer = TimestampSigner()
    try:
        payload = signer.unsign(token, max_age=settings.WALDUR_CORE['INVITATION_MAX_AGE'])
    except BadSignature:
        raise serializers.ValidationError('Invalid signature.')

    parts = payload.split('.')
    if len(parts) != 2:
        raise serializers.ValidationError('Invalid payload.')

    user_uuid = parts[0]
    invitation_uuid = parts[1]

    if not core_utils.is_uuid_like(user_uuid):
        raise serializers.ValidationError('Invalid user UUID.')

    try:
        user = core_models.User.objects.filter(uuid=parts[0], is_active=True, is_staff=True).get()
    except core_models.User.DoesNotExist:
        raise serializers.ValidationError('Invalid user UUID.')

    if not core_utils.is_uuid_like(invitation_uuid):
        raise serializers.ValidationError('Invalid invitation UUID.')

    try:
        invitation = models.Invitation.objects.get(uuid=parts[1], state=models.Invitation.State.REQUESTED)
    except models.Invitation.DoesNotExist:
        raise serializers.ValidationError('Invalid invitation UUID.')

    return user, invitation


def normalize_username(username):
    return ''.join(c if c.isalnum() else '_' for c in username.lower())


def generate_safe_username(username):
    username = generate_username(username)
    # Maximum length for FreeIPA username is 32 chars
    if len(username) > 32:
        prefix_length = len(settings.WALDUR_FREEIPA['USERNAME_PREFIX'])
        username = generate_username(pwgen(32 - prefix_length))
    return username


@transaction.atomic
def get_or_create_user(invitation):
    if invitation.civil_number:
        user = core_models.User.objects.filter(civil_number=invitation.civil_number).first()
        if user:
            return user, False

    user = core_models.User.objects.filter(email=invitation.email).first()
    if user:
        return user, False

    username = normalize_username(invitation.email)
    user = core_models.User.objects.filter(username=username).first()
    if user:
        return user, False

    payload = {
        field: getattr(invitation, field)
        for field in (
            'full_name', 'native_name',
            'organization', 'civil_number',
            'job_title', 'phone_number',
        )
    }
    user = core_models.User.objects.create_user(
        username=username,
        email=invitation.email,
        registration_method='FREEIPA',
        **payload
    )
    user.set_unusable_password()
    user.save()
    return user, True


@transaction.atomic
def get_or_create_profile(user, username, password):
    profile = Profile.objects.filter(user=user).first()
    if profile:
        return profile, False

    profile = Profile.objects.create(
        user=user,
        username=username,
    )
    try:
        FreeIPABackend().create_profile(profile, password=password)
        tasks.schedule_sync()
    except freeipa_exceptions.DuplicateEntry:
        pass
    return profile, True



from django.apps import AppConfig
from django.db.models import signals


class BillingConfig(AppConfig):
    name = 'waldur_mastermind.billing'
    verbose_name = 'Billing'

    def ready(self):
        from waldur_mastermind.invoices import models as invoices_models

        from . import handlers, models

        for index, model in enumerate(models.PriceEstimate.get_estimated_models()):
            signals.post_save.connect(
                handlers.create_price_estimate,
                sender=model,
                dispatch_uid='waldur_mastermind.billing.'
                             'create_price_estimate_%s_%s' % (index, model.__class__),
            )

        for index, model in enumerate(models.PriceEstimate.get_estimated_models()):
            signals.pre_delete.connect(
                handlers.delete_stale_price_estimate,
                sender=model,
                dispatch_uid='waldur_mastermind.billing.'
                             'delete_stale_price_estimate_%s_%s' % (index, model.__class__),
            )

        signals.post_save.connect(
            handlers.update_estimate_when_invoice_is_created,
            sender=invoices_models.Invoice,
            dispatch_uid='waldur_mastermind.billing.'
                         'update_estimate_when_invoice_is_created',
        )

        signals.post_save.connect(
            handlers.process_invoice_item,
            sender=invoices_models.InvoiceItem,
            dispatch_uid='waldur_mastermind.billing. process_invoice_item',
        )

        signals.post_save.connect(
            handlers.log_price_estimate_limit_update,
            sender=models.PriceEstimate,
            dispatch_uid='waldur_mastermind.billing.log_price_estimate_limit_update',
        )



import logging

from rest_framework import decorators, response

from waldur_ansible.common import serializers as common_serializers
from waldur_ansible.python_management import views as python_management_views
from waldur_ansible.python_management import serializers as python_management_serializers

from waldur_core.core import views as core_views, managers as core_managers, mixins as core_mixins, models as core_models
from . import models, serializers, executors, jupyter_hub_management_service

jupyter_hub_management_requests_models = [models.JupyterHubManagementSyncConfigurationRequest,
                                          models.JupyterHubManagementDeleteRequest,
                                          models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest,
                                          models.JupyterHubManagementMakeVirtualEnvironmentLocalRequest]

logger = logging.getLogger(__name__)


class JupyterHubManagementViewSet(core_mixins.AsyncExecutor, core_views.ActionsViewSet):
    lookup_field = 'uuid'
    queryset = models.JupyterHubManagement.objects.all().order_by('pk')
    serializer_class = serializers.JupyterHubManagementSerializer
    python_management_request_executor = executors.JupyterHubManagementRequestExecutor
    service = jupyter_hub_management_service.JupyterHubManagementService()

    def retrieve(self, request, *args, **kwargs):
        jupyter_hub_management = self.get_object()
        jupyter_hub_management_serializer = self.get_serializer(jupyter_hub_management)

        requests = self.find_related_requests(jupyter_hub_management)
        python_management_requests = self.find_related_python_management_requests(jupyter_hub_management)

        python_management = python_management_serializers.PythonManagementSerializer(jupyter_hub_management.python_management, context={'request': request})

        return response.Response(
            {'jupyter_hub_management': jupyter_hub_management_serializer.data,
             'requests': requests,
             'python_management_requests': python_management_requests,
             'selected_python_management': python_management.data})

    def find_related_python_management_requests(self, jupyter_hub_management):
        # we are interested in blocking python management requests (currently running)
        python_management_requests = core_managers.SummaryQuerySet(python_management_views.python_management_requests_models) \
            .filter(python_management=jupyter_hub_management.python_management) \
            .exclude(state__in=[core_models.StateMixin.States.OK, core_models.StateMixin.States.ERRED]) \
            .order_by("-created")
        python_management_requests_serializer = common_serializers.SummaryApplicationSerializer(
            python_management_requests, many=True, context={'select_output': False})
        return python_management_requests_serializer.data

    def find_related_requests(self, jupyter_hub_management):
        jupyter_hub_management_requests = core_managers.SummaryQuerySet(jupyter_hub_management_requests_models).filter(
            jupyter_hub_management=jupyter_hub_management).order_by("-created")
        jupyter_hub_management_requests_serializer = serializers.SummaryJupyterHubManagementRequestsSerializer(
            jupyter_hub_management_requests, many=True, context={'select_output': False})
        return jupyter_hub_management_requests_serializer.data

    @core_mixins.ensure_atomic_transaction
    def perform_create(self, serializer):
        # user cannot create jupyter management if python management has not been created
        jupyter_hub_management = serializer.save()

        self.service.execute_sync_configuration_request_if_allowed(jupyter_hub_management)

        for virtual_env in serializer.validated_data.get('updated_virtual_environments'):
            if virtual_env['jupyter_hub_global']:
                virtual_env_request = models.JupyterHubManagementMakeVirtualEnvironmentGlobalRequest(
                    jupyter_hub_management=jupyter_hub_management, virtual_env_name=virtual_env['name'])
                virtual_env_request.save()
                executors.JupyterHubManagementRequestExecutor.execute(virtual_env_request, async=self.async_executor)

    @core_mixins.ensure_atomic_transaction
    def perform_update(self, serializer):
        persisted_jupyter_hub_management = self.get_object()
        incoming_validated_data = serializer.validated_data

        if self.service.has_jupyter_hub_config_changed(incoming_validated_data, persisted_jupyter_hub_management) \
                or self.service.is_last_sync_request_erred(persisted_jupyter_hub_management):
            persisted_jupyter_hub_management = serializer.save()
            self.service.execute_sync_configuration_request_if_allowed(persisted_jupyter_hub_management)

        self.service.issue_localize_globalize_requests(persisted_jupyter_hub_management, serializer.validated_data)

    @core_mixins.ensure_atomic_transaction
    def perform_destroy(self, persisted_jupyter_hub_management):
        self.service.schedule_jupyter_hub_management_removal(persisted_jupyter_hub_management)

    @decorators.detail_route(url_path="requests/(?P<request_uuid>.+)", methods=['get'])
    def find_request_with_output_by_uuid(self, request, uuid=None, request_uuid=None):
        requests = core_managers.SummaryQuerySet(jupyter_hub_management_requests_models).filter(
            jupyter_hub_management=self.get_object(), uuid=request_uuid)
        serializer = serializers.SummaryJupyterHubManagementRequestsSerializer(requests, many=True, context={'select_output': True})
        return response.Response(serializer.data)



from __future__ import unicode_literals

from datetime import timedelta

from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from waldur_core.core.serializers import GenericRelatedField
from waldur_core.structure.models import Customer, Project


class DailyHistoryQuotaSerializer(serializers.Serializer):
    scope = GenericRelatedField(related_models=(Project, Customer))
    quota_names = serializers.ListField(child=serializers.CharField(), required=False)
    start = serializers.DateField(format='%Y-%m-%d', required=False)
    end = serializers.DateField(format='%Y-%m-%d', required=False)

    def validate(self, attrs):
        if 'quota_names' not in attrs:
            attrs['quota_names'] = attrs['scope'].get_quotas_names
        if 'end' not in attrs:
            attrs['end'] = timezone.now().date()
        if 'start' not in attrs:
            attrs['start'] = timezone.now().date() - timedelta(days=30)
        if attrs['start'] >= attrs['end']:
            raise serializers.ValidationError(
                _('Invalid period specified. `start` should be lesser than `end`.')
            )
        return attrs



from cStringIO import StringIO

from PIL import Image
from django.conf import settings

from waldur_ansible.playbook_jobs import tasks


def delete_playbook_workspace(sender, instance, **kwargs):
    if not settings.WALDUR_PLAYBOOK_JOBS.get('PRESERVE_PLAYBOOK_WORKSPACE_AFTER_DELETION', False):
        tasks.delete_playbook_workspace.delay(instance.workspace)


def resize_playbook_image(sender, instance, **kwargs):
    if not instance.tracker.has_changed('image') or not instance.image:
        return

    size = settings.WALDUR_PLAYBOOK_JOBS['PLAYBOOK_ICON_SIZE']
    field = instance.image
    image = Image.open(field.file)
    image.thumbnail(size, Image.ANTIALIAS)

    image_file = StringIO()
    image.save(image_file, 'png')
    field.file = image_file



from decimal import Decimal
import logging

from django.db import transaction
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers

from waldur_core.core import serializers as core_serializers
from waldur_core.media.serializers import ProtectedFileField
from waldur_core.structure.models import VATException

from . import models


logger = logging.getLogger(__name__)


class PaymentSerializer(core_serializers.AugmentedSerializerMixin,
                        serializers.HyperlinkedModelSerializer):

    amount = serializers.DecimalField(max_digits=9, decimal_places=2)
    state = serializers.ReadOnlyField(source='get_state_display')
    return_url = serializers.CharField(write_only=True)
    cancel_url = serializers.CharField(write_only=True)

    class Meta(object):
        model = models.Payment

        fields = (
            'url', 'uuid', 'created', 'modified', 'state',
            'amount', 'customer', 'return_url', 'cancel_url', 'approval_url', 'error_message', 'tax'
        )

        read_only_fields = ('approval_url', 'error_message', 'tax')
        protected_fields = ('customer', 'amount', 'return_url', 'cancel_url')

        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'paypal-payment-detail'},
            'customer': {'lookup_field': 'uuid', 'view_name': 'customer-detail'},
        }

    def create(self, validated_data):
        customer = validated_data['customer']
        amount = validated_data['amount']

        try:
            rate = customer.get_vat_rate() or 0
        except (NotImplemented, VATException) as e:
            rate = 0
            logger.warning('Unable to compute VAT rate for customer with UUID %s, error is %s',
                           customer.uuid, e)
        validated_data['tax'] = Decimal(rate) / Decimal(100) * amount

        return super(PaymentSerializer, self).create(validated_data)


class PaymentApproveSerializer(serializers.Serializer):
    payment_id = serializers.CharField()
    payer_id = serializers.CharField()
    token = serializers.CharField()


class PaymentCancelSerializer(serializers.Serializer):
    token = serializers.CharField()


class InvoiceItemSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.InvoiceItem
        fields = ('price', 'tax', 'unit_price', 'quantity', 'unit_of_measure', 'name', 'start', 'end')


class InvoiceSerializer(core_serializers.AugmentedSerializerMixin,
                        serializers.HyperlinkedModelSerializer):

    pdf = ProtectedFileField(read_only=True)
    items = InvoiceItemSerializer(many=True, read_only=True)
    payment_url = serializers.SerializerMethodField()
    issuer_details = serializers.JSONField()
    customer_details = serializers.JSONField(source='payment_details')

    class Meta(object):
        model = models.Invoice
        fields = (
            'url', 'uuid', 'total', 'price', 'tax', 'pdf', 'backend_id', 'issuer_details',
            'invoice_date', 'end_date', 'state', 'items', 'payment_url', 'customer_details',
            'customer', 'customer_uuid', 'customer_name', 'year', 'month', 'number',
        )
        related_paths = ('customer',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'paypal-invoice-detail'},
            'customer': {'lookup_field': 'uuid'}
        }

    def get_payment_url(self, invoice):
        backend = invoice.get_backend()
        return backend.get_payment_view_url(invoice.backend_id) if invoice.backend_id else None


class InvoiceUpdateWebHookSerializer(serializers.Serializer):

    @transaction.atomic()
    def save(self, **kwargs):
        backend_id = self.initial_data['resource']['id']
        status = self.initial_data['resource']['status']

        try:
            invoice = models.Invoice.objects.get(backend_id=backend_id)
        except models.Invoice.DoesNotExist:
            raise serializers.ValidationError({'backend_id': _('Invoice with id "%s" cannot be found.') % backend_id})

        invoice.state = status
        invoice.save(update_fields=['state'])
        return invoice



ADMIN_INSTALLED_APPS = (
    'fluent_dashboard',
    'admin_tools',
    'admin_tools.theming',
    'admin_tools.menu',
    'admin_tools.dashboard',
    'django.contrib.admin',
)

ADMIN_TEMPLATE_LOADERS = (
    'admin_tools.template_loaders.Loader',  # required by django-admin-tools >= 0.7.0
)

FLUENT_DASHBOARD_APP_ICONS = {
    'core/user': 'system-users.png',
    'structure/customer': 'system-users.png',
    'structure/servicesettings': 'preferences-other.png',
    'structure/project': 'folder.png',
    'structure/projectgroup': 'folder-bookmark.png',
    'backup/backup': 'document-export-table.png',
    'backup/backupschedule': 'view-resource-calendar.png',
    'cost_tracking/pricelistitem': 'view-bank-account.png',
    'cost_tracking/priceestimate': 'feed-subscribe.png',
    'cost_tracking/defaultpricelistitem': 'view-calendar-list.png'
}

ADMIN_TOOLS_INDEX_DASHBOARD = 'waldur_core.server.admin.dashboard.CustomIndexDashboard'
ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'waldur_core.server.admin.dashboard.CustomAppIndexDashboard'
ADMIN_TOOLS_MENU = 'waldur_core.server.admin.menu.CustomMenu'

# Should be specified, otherwise all Applications dashboard will be included.
FLUENT_DASHBOARD_APP_GROUPS = ()

FLUENT_DASHBOARD_QUICK_ACCESS_LINKS = [
    # ['Waldur cloud management service', 'https://waldur.com'],
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-01-29 15:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_azure', '0004_storageaccount'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='image',
            options={'ordering': ['publisher', 'name', 'sku']},
        ),
        migrations.AlterModelOptions(
            name='location',
            options={'ordering': ['name']},
        ),
        migrations.AlterUniqueTogether(
            name='image',
            unique_together=set([]),
        ),
        migrations.AlterModelOptions(
            name='size',
            options={'ordering': ['number_of_cores', 'memory_in_mb']},
        ),
        migrations.AlterUniqueTogether(
            name='image',
            unique_together=set([('settings', 'backend_id')]),
        ),
        migrations.RemoveField(
            model_name='image',
            name='offer',
        ),
    ]



from django.apps import AppConfig
from django.db.models import signals


class SupportInvoicesConfig(AppConfig):
    name = 'waldur_mastermind.support_invoices'
    verbose_name = 'Support'

    def ready(self):
        from waldur_mastermind.invoices import registrators
        from waldur_mastermind.support import models as support_models
        from waldur_mastermind.marketplace import models as marketplace_models
        from . import handlers, registrators as support_registrators

        registrators.RegistrationManager.add_registrator(
            support_models.Offering,
            support_registrators.OfferingRegistrator
        )

        signals.post_save.connect(
            handlers.add_new_offering_to_invoice,
            sender=marketplace_models.OrderItem,
            dispatch_uid='support_invoices.handlers.add_new_offering_to_invoice',
        )

        signals.post_save.connect(
            handlers.terminate_invoice_when_offering_cancelled,
            sender=support_models.Offering,
            dispatch_uid='support_invoices.handlers.terminate_invoice_when_offering_cancelled',
        )

        signals.pre_delete.connect(
            handlers.terminate_invoice_when_offering_deleted,
            sender=support_models.Offering,
            dispatch_uid='support_invoices.handlers.terminate_invoice_when_offering_deleted',
        )

        signals.post_save.connect(
            handlers.switch_plan_resource,
            sender=marketplace_models.Resource,
            dispatch_uid='support_invoices.handlers.switch_plan_resource',
        )

        signals.post_save.connect(
            handlers.add_component_usage,
            sender=marketplace_models.ComponentUsage,
            dispatch_uid='support_invoices.handlers.add_component_usage',
        )

        signals.post_save.connect(
            handlers.add_new_offering_details_to_invoice,
            sender=support_models.Offering,
            dispatch_uid='waldur_mastermind.invoices.add_new_offering_details_to_invoice',
        )

        signals.pre_delete.connect(
            handlers.update_invoice_on_offering_deletion,
            sender=support_models.Offering,
            dispatch_uid='waldur_mastermind.invoices.update_invoice_on_offering_deletion',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import django_fsm
import model_utils.fields

import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('structure', '0003_order_by_name'),
        ('marketplace', '0044_cartitem'),
    ]

    operations = [
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('attributes', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.ContentType')),
                ('plan', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marketplace.Plan')),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='structure.Project')),
                ('limits', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('state', django_fsm.FSMIntegerField(choices=[(1, 'Creating'), (2, 'OK'), (3, 'Erred'), (4, 'Updating'), (5, 'Terminating'), (6, 'Terminated')], default=1)),
                ('offering', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='marketplace.Offering')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified'))
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='componentquota',
            name='resource',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marketplace.Resource'),
        ),
        migrations.AddField(
            model_name='componentusage',
            name='resource',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marketplace.Resource'),
        ),
        migrations.AddField(
            model_name='orderitem',
            name='resource',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='marketplace.Resource'),
        ),
        migrations.AddField(
            model_name='orderitem',
            name='limits',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from django.db.models import Count


def find_duplicates(model):
    return model.objects\
                .values('settings_id', 'backend_id')\
                .annotate(Count('pk'))\
                .filter(pk__count__gt=1)


def get_duplicate_set(model, item):
    affected = model.objects.filter(
        settings_id=item['settings_id'],
        backend_id=item['backend_id']
    )
    chosen = affected.first()
    duplicates = affected.exclude(pk=chosen.pk)
    return chosen, duplicates


def drop_duplicate_security_groups(apps, schema_editor):
    SecurityGroup = apps.get_model('openstack_tenant', 'SecurityGroup')

    for item in find_duplicates(SecurityGroup):
        securitygroup, duplicates = get_duplicate_set(SecurityGroup, item)
        SecurityGroup.instances.through.objects\
            .filter(securitygroup__in=duplicates)\
            .update(securitygroup=securitygroup)
        duplicates.delete()


def drop_duplicate_networks(apps, schema_editor):
    Network = apps.get_model('openstack_tenant', 'Network')
    SubNet = apps.get_model('openstack_tenant', 'SubNet')

    for item in find_duplicates(Network):
        network, duplicates = get_duplicate_set(Network, item)
        SubNet.objects.filter(network__in=duplicates).update(network=network)
        duplicates.delete()


def drop_duplicate_subnets(apps, schema_editor):
    SubNet = apps.get_model('openstack_tenant', 'SubNet')
    InternalIP = apps.get_model('openstack_tenant', 'InternalIP')

    for item in find_duplicates(SubNet):
        subnet, duplicates = get_duplicate_set(SubNet, item)
        InternalIP.objects.filter(subnet__in=duplicates).update(subnet=subnet)
        duplicates.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0027_remove_duplicate_floating_ips'),
    ]

    operations = [
        migrations.RunPython(drop_duplicate_security_groups, elidable=True),
        migrations.RunPython(drop_duplicate_networks, elidable=True),
        migrations.RunPython(drop_duplicate_subnets, elidable=True),
    ]



from __future__ import unicode_literals

import logging

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel

from waldur_core.core.models import UuidMixin
from waldur_core.structure import models as structure_models
from waldur_core.structure.models import NewResource

logger = logging.getLogger(__name__)


class RancherService(structure_models.Service):
    projects = models.ManyToManyField(
        structure_models.Project, related_name='rancher_services', through='RancherServiceProjectLink')

    class Meta:
        unique_together = ('customer', 'settings')
        verbose_name = _('Rancher provider')
        verbose_name_plural = _('Rancher providers')

    @classmethod
    def get_url_name(cls):
        return 'rancher'


class RancherServiceProjectLink(structure_models.ServiceProjectLink):

    service = models.ForeignKey(RancherService)

    class Meta(structure_models.ServiceProjectLink.Meta):
        verbose_name = _('Rancher provider project link')
        verbose_name_plural = _('Rancher provider project links')

    @classmethod
    def get_url_name(cls):
        return 'rancher-spl'


@python_2_unicode_compatible
class Cluster(NewResource):
    service_project_link = models.ForeignKey(
        RancherServiceProjectLink, related_name='k8s_clusters', on_delete=models.PROTECT)

    """
    Rancher generated node installation command base. For example:
    sudo docker run -d --privileged --restart=unless-stopped --net=host
    -v /etc/kubernetes:/etc/kubernetes -v /var/run:/var/run rancher/rancher-agent:v2.2.8
    --server https://192.168.33.13
    --token df8vrttmcmz8qzfbp74t6nkl5t5pbkrjh8wgkv27zrk8ldhfj6sp4w
    --ca-checksum e3596989da2fa5f8a7bdfbfd1079f87033217152db4dfc93532932b17aad1567
    --etcd --controlplane --worker
    """
    node_command = models.CharField(max_length=1024, blank=True,
                                    help_text='Rancher generated node installation command base.')

    class Meta(object):
        unique_together = (('service_project_link', 'backend_id'), ('service_project_link', 'name'))

    @classmethod
    def get_url_name(cls):
        return 'rancher-cluster'

    def __str__(self):
        return self.name


class Node(TimeStampedModel, UuidMixin):
    content_type = models.ForeignKey(ContentType, null=True, related_name='+')
    object_id = models.PositiveIntegerField(null=True)
    instance = GenericForeignKey('content_type', 'object_id')  # a virtual machine where will deploy k8s node.
    cluster = models.ForeignKey(Cluster, on_delete=models.CASCADE)

    class Meta(object):
        unique_together = ('content_type', 'object_id')

    class Permissions(object):
        customer_path = 'cluster__service_project_link__project__customer'
        project_path = 'cluster__service_project_link__project'
        service_path = 'cluster__service_project_link__service'

    @classmethod
    def get_url_name(cls):
        return 'rancher-node'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-22 16:50
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('marketplace', '0053_add_category_component'),
    ]

    operations = [
        migrations.CreateModel(
            name='CategoryComponentUsage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('date', models.DateField()),
                ('reported_usage', models.PositiveIntegerField(null=True)),
                ('fixed_usage', models.PositiveIntegerField(null=True)),
                ('component', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='marketplace.CategoryComponent')),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from rest_framework import serializers

from waldur_mastermind.marketplace import processors
from waldur_mastermind.marketplace.plugins import manager
from waldur_mastermind.marketplace_slurm import PLUGIN_NAME
from waldur_slurm import views as slurm_views


class CreateAllocationProcessor(processors.BaseCreateResourceProcessor):
    viewset = slurm_views.AllocationViewSet

    fields = (
        'name',
        'description',
    )

    def get_post_data(self):
        if not self.order_item.limits:
            raise serializers.ValidationError('Quota are not defined.')

        payload = super(CreateAllocationProcessor, self).get_post_data()

        for component_type in manager.get_component_types(PLUGIN_NAME):
            try:
                limit = self.order_item.limits[component_type]
            except KeyError:
                raise serializers.ValidationError('%s component quota is not defined' % component_type)
            else:
                payload[component_type + '_limit'] = limit
        return payload


class DeleteAllocationProcessor(processors.DeleteResourceProcessor):
    viewset = slurm_views.AllocationViewSet



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-01 08:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0042_order__file'),
    ]

    operations = [
        migrations.AlterField(
            model_name='offering',
            name='shared',
            field=models.BooleanField(default=True, help_text='Accessible to all customers.'),
        ),
    ]



from __future__ import unicode_literals

from waldur_core.core import WaldurExtension


class RancherExtension(WaldurExtension):

    class Settings:
        WALDUR_RANCHER = {}

    @staticmethod
    def django_app():
        return 'waldur_rancher'

    @staticmethod
    def rest_urls():
        from .urls import register_in
        return register_in
